use simplelog::info;
use crate::mappers::Mapper;
use crate::rom::{NametableMirroring, Rom};

const PRG_BANK_SIZE: usize = 0x4000;
const CHR_BANK_SIZE: usize = 0x2000;

pub struct NRom {
    rom: Rom,
}

impl NRom {
    /// Creates an instance of the NRom mapper.
    ///
    /// # Arguments
    ///  * rom - The ROM
    pub fn new(rom: Rom) -> Self {
        info!("Using NRom mapper");
        Self { rom }
    }
}

impl Mapper for NRom {
    fn get_prg_byte(&self, addr: u16) -> u8 {
        let addr_usize = addr as usize;
        if addr_usize < PRG_BANK_SIZE || self.rom.prg_rom_size > PRG_BANK_SIZE {
            return self.rom.prg_rom[addr_usize];
        }

        self.rom.prg_rom[addr_usize - PRG_BANK_SIZE]
    }

    fn get_chr_byte(&self, addr: u16) -> u8 {
        let addr_usize = addr as usize;

        assert!(addr_usize < CHR_BANK_SIZE);
        self.rom.chr_rom[addr_usize]
    }

    fn are_nametables_mirrored(&self) -> NametableMirroring {
        self.rom.nametable_mirroring
    }
}
